import 'package:args/command_runner.dart';
import 'package:dshell/dshell.dart';
import 'package:dshell/dshell.dart' as d;
import 'package:nginx_le_shared/nginx_le_shared.dart';

import '../command_exeption.dart';
import 'util.dart';

class AcquireCommand extends Command<void> {
  @override
  String get description =>
      'Obtains or forces the renewal of a lets encrypt certificate';

  @override
  String get name => 'acquire';

  AcquireCommand() {
    argParser.addFlag('ask',
        abbr: 'a',
        defaultsTo: false,
        help:
            '--ask prompt the user for the namecheap apikey and apiusername.');

    argParser.addFlag('env',
        abbr: 'e',
        defaultsTo: false,
        help:
            '''--env expect the namecheap apikey and apiusername to be present as environment variables.
        export NAMECHEAP_API_KEY=xxxxxx
        export NAMECHEAP_API_USERNAME=yyyy
        ''');

    argParser.addOption('containerid',
        abbr: 'c',
        help:
            'The docker containerid to attach to in the form --containerid="XXXXX"');
    argParser.addOption('name',
        abbr: 'n', help: 'The name of the docker container to attach to');
    argParser.addFlag('debug',
        abbr: 'd', help: 'Outputs additional logging information');

    argParser.addFlag('staging',
        abbr: 's', help: 'Acquire a staging (test) certificate.');
  }

  @override
  void run() {
    print('acquire command running');

    var debug = argResults['debug'] as bool;

    if (debug) Settings().setVerbose(enabled: debug);

    var ask = argResults['ask'] as bool;
    var env = argResults['env'] as bool;
    var containerid = argResults['containerid'] as String;
    var name = argResults['name'] as String;
    var staging = argResults['staging'] as bool;

    if (name != null && containerid != null) {
      printerr('You may only pass one of "name" and "containerid');
      showUsage(argParser);
    }

    if (ask == true && env == true) {
      printerr('You may only pass one of "ask" and "env');
      showUsage(argParser);
    }

    if (ask == false && env == false) {
      printerr('You may pass either of "ask" or "env');
      showUsage(argParser);
    }

    String apiKey;
    String apiUsername;
    if (ask) {
      apiKey = d.ask(
          prompt: 'Enter the NameCheap apiKey:',
          validator: Ask.required,
          hidden: true);
      apiUsername = d.ask(
          prompt: 'Enter the NameCheap apiUsername:', validator: Ask.required);
    } else {
      _validateEnv(NAMECHEAP_API_KEY);
      _validateEnv(NAMECHEAP_API_USER);
      apiKey = d.env('NAMECHEAP_API_KEY');
      apiUsername = d.env('NAMECHEAP_API_USER');
    }

    var target = containerOrName(argParser, argResults);

    var cmd =
        'docker exec -it $target /home/bin/acquire --$NAMECHEAP_API_KEY=$apiKey --$NAMECHEAP_API_USER=$apiUsername';

    if (staging == true) cmd += ' --staging';
    if (debug == true) cmd += ' --debug';
    cmd.run;
  }

  String _validateEnv(String envName, {String defaultTo}) {
    var value = env(envName);

    value ??= defaultTo;

    if (value == null) {
      throw CommandException(
          'You must passed the environment variable $envName');
    }
    return value;
  }
}
