import 'dart:io';

import 'package:args/command_runner.dart';
import 'package:dshell/dshell.dart';
import 'package:nginx_le_shared/nginx_le_shared.dart';

import '../../external_run_config.dart';

/// Starts nginx and the certbot scheduler.
class StartCommand extends Command<void> {
  @override
  String get description => 'starts the ngix server';

  @override
  String get name => 'start';

  StartCommand() {
    argParser.addOption('fqdn',
        abbr: 'f',
        help:
            'The fully qualified domain name of your webserver --fqdn="www.example.com.au"');
    argParser.addOption('tld',
        abbr: 't',
        help: 'The top level domain of the fqdn e.g. --tld="com.au"');
    argParser.addOption('mode',
        abbr: 'm', defaultsTo: 'public', help: '''--mode=<public|private>
    Sets the mode of the server. 
    public - server is exposed to the internet. 
    private - server is an internal an not exposed via the internet ''');

    argParser.addOption('image',
        abbr: 'i',
        help: 'The docker image name in the form --image="repo/image:version"');
    argParser.addOption('emailaddress',
        abbr: 'e', help: 'The email address to send notifications to.');
    argParser.addOption('name',
        abbr: 'n', help: 'Optionally names the resulting docker container');
    argParser.addFlag('debug',
        defaultsTo: false,
        abbr: 'd',
        help: 'Outputs additional logging information');
  }

  @override
  void run() {
    print('nginx-le container is starting.');

    var fqdn = argResults['fqdn'] as String;
    var tld = argResults['tld'] as String;
    var mode = argResults['mode'] as String;
    var image = argResults['image'] as String;
    var emailaddress = argResults['emailaddress'] as String;
    var name = argResults['name'] as String;

    if (fqdn == null) {
      print('You must pass an fqdn --fqdn=www.example.com.au');
      showUsage(argParser);
    }

    if (tld == null) {
      print('You must pass a tld --tld=com.au');
      showUsage(argParser);
    }
    if (image == null) {
      print('You must pass a image --image=repo/image:version');
      showUsage(argParser);
    }

    if (mode == null) {
      print('You must pass a mode --mode=<public|private>');
      showUsage(argParser);
    }

    if (emailaddress == null) {
      print(
          'You must pass an emailaddress --emailaddress=notifications@example.com');
      showUsage(argParser);
    }

    var debug = argResults['debug'] as bool;
    Settings().setVerbose(enabled: debug);

    print(
        'Starting nginx fqnd=$fqdn tld=$tld mode=$mode emailaddress=$emailaddress debug=$debug');
    var containerName = '';
    if (name != null) {
      containerName = '--name=$name';
    }

    ExternalRunConfig(
            fqdn: fqdn,
            tld: tld,
            emailaddress: emailaddress,
            mode: mode,
            image: image,
            name: name)
        .save();

    'docker run'
            ' $containerName'
            ' --env=FQDN=$fqdn'
            ' --env=TLD=$tld'
            ' --env=MODE=$mode'
            ' --env=EMAIL_ADDRESS=$emailaddress'
            ' --env=DEBUG=$debug'
            ' --net=host'
            ' -v certificates:${Certbot.letsEncryptRootPath}'
            ' $image'
        .run;
  }

  void showUsage(ArgParser parser) {
    print(parser.usage);
    exit(-1);
  }
}
